/*
 * Canvas Game Library
 * Module: Image
 * 
 * Copyright Chainsawrus Wreaks 2011
 * All rights reserved.
 */

/**
 * @fileOverview This is the class that handles all the graphics derived from images. It gives the developer
 * the methods to load and manipulate imagedata as well as giving the user animation possibilites.
 */

/**
 * AJAX module namespace
 * @namespace CGL.image
 */
CGL.image = {
	
	Image : function(){
		var image = {
			image : new Image(),
			useAlpha : false,
			alpha : 1.0,
			
			load : function(src) {
				CGL.resources.addImageResource();
				image.image.addEventListener("load", CGL.resources.notifyLoadedImage, false);
				image.image.src = src;
			},
			
			render : function(ctx, x, y) {
				if(image.useAlpha)
				{
					ctx.globalAlpha = image.alpha;
				}
				ctx.drawImage(image.image, x.toFixed(), y.toFixed());
				if(image.useAlpha)
				{
					ctx.globalAlpha = 1.0;
				}
			},
			
			renderSpec : function(ctx, srcx, srcy, srcwidth, srcheight, destx, desty, destwidth, destheight) {
				if(image.useAlpha)
				{
					ctx.globalAlpha = image.alpha;
				}
				ctx.drawImage(image.image, srcx, srcy, srcwidth, srcheight, destx, desty, destwidth, destheight);
				if(image.useAlpha)
				{
					ctx.globalAlpha = 1.0;
				}
			},
			
			setAlpha : function(alpha) {
				image.alpha = alpha;
				if(alpha != 1.0)
				{
					image.useAlpha = true;
				}
				else
				{
					image.useAlpha = false;
				}
			}
		};
		return image;
	}
};

CGL.image.animation = {

	Frame : function() {
		var frame = {
			image : new CGL.image.Image(),
			duration : 0,
			
			load : function(src) {
				frame.image.load(src);
			},
			
			setDuration : function(time) {
				frame.duration = time;
			},
			
			getDuration : function() {
				return frame.duration;
			},
			
			render : function(ctx, x, y, alpha) {
				frame.image.render(ctx, x, y);
			}
		};
		return frame;
	},
		
	Animation : function(){
		var animation = {
			active : true,
			activeFrame : 0,
			maxFrames : 0,
			images : new Array(),
			timeCount : 0,
			loop : true,
			pingpong : false,
				
			load : function(frames, time) {
				var temp;
					
				for(var i = 0; i < frames.length; i++)
				{
					temp = new CGL.image.animation.Frame();
					temp.load(frames[i]);
					temp.setDuration(time);
					animation.images.push(temp);
				}
				animation.maxFrames = animation.images.length - 1;
			},
				
			nextFrame : function(){
				animation.activeFrame += 1;
			
				if(animation.activeFrame > animation.maxFrames)
				{
					if(animation.loop)
					{
						animation.activeFrame = 0;
					}
					else
					{
						animation.activeFrame = 0;
						animation.active = false;
					}
				}
			},
				
			setGlobalSpeed : function(time){
				for(var i = 0; i < animation.images.length; i++)
				{
					animation.images[i].setDuration(time);
				}
			},
			
			setSpeed : function(frame, time) {
				animation.images[frame].setDuration(time);
			},
				
			setLoop : function(looping){
				animation.loop = looping;
			},
				
			setActiveFrame : function(actFrame){
				animation.activeFrame = actFrame;
			},
				
			stop : function(){
				animation.active = false;
			},
				
			start : function(){
				animation.active = true;
			},
				
			replay : function(){
				animation.activeFrame = 0;
				animation.active = true;
			},
				
			update : function(time){
				if(animation.active)
				{
					animation.timeCount += time;
				}
						
				if(animation.timeCount > animation.images[animation.activeFrame].getDuration())
				{
					animation.nextFrame();
					animation.timeCount = 0;
				}
			},
				
			render : function(ctx, x, y, alpha){
				animation.images[animation.activeFrame].render(ctx, x, y);
			}
		};
		return animation;
	}	
};